////////////////////////////////////////
// 49104 v.3 opg.1
//
// Formatering af tal i et andet tal-system.
// Denne klasse foretager selve konverteringen.
//

public class BaseConvert
{
	// Den gldende base for udskrivning
	private int fbase = 10;
	// Oversttelse mellem cifre og talvrdier
	private char digits[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z'};

	
	// Generer et nyt BaseConvert objekt med base angivet
	public BaseConvert( int base )
	{
		setBase( base );
	}
	
	
	// Generer et nyt BaseConvert objekt med standard base-10
	public BaseConvert()
	{
		fbase = 10;
	}
	
	
	// Streng-representation af objekt
	public String toString()
	{
		return "[BaseConvert]: (Base number conversion routines) base-"+ fbase;
	}
	
	
	// Returner gldende base
	public int getBase()
	{
		return fbase;
	}

	// St basen for konverterings rutinerne (2->35)
	public void setBase(int base)
	{
		if ((base > 1) && (base < 36))
		{
			fbase = base;
		}

	}

	// Konverter long til streng med den aktive base
	//   Algoritmen fungerer bagfra:
	//   - Tag rest af division med base, 
	//   - Skrive cifret (findes ved opslag i tabel) foran alle andre cifre
	//   - Dividere med base.
	//   - Fortst indtil tal er =0
	public String toString(long value)
	{
		int digit = 0;
		String output = "";

		do 
		{
			digit = (int) (value % fbase);
			value /= fbase;
			output = digits[digit] + output;
		} while (value != 0);

		return output;
	}
	
	
	// Konverter streng til long:
	//  Algoritmen fungerer forfra - 
	//  - Gang nuvrende vrdi op med basen,
	//  - Tag forreste ciffer's talvrdi og adder det til tallet
	//  - Fortst for alle cifre
	//
	// NB! rutinen kan ligesom Integer.parseInt lave en NumberFormatException - se
	//     Dietel&Dietel for en forklaring p at fange exceptions 
	//     try { s = b.parseString("a2003"); } catch (NumberFormatException e) { //fejl };
	public long parseString( String s ) throws NumberFormatException
	{
		long val = 0;
		char c = 0;
		int l = s.length();
		boolean found = false;
		
		for (int strcnt=0; strcnt<l; strcnt++)
		{
			val *= fbase;
			c = Character.toUpperCase( s.charAt(strcnt) );
			found = false;
			for (int chloc=0; chloc<digits.length; chloc++)
				if (digits[chloc]==c)
				{
					if (chloc>=fbase) throw new NumberFormatException( "Character \""+c+"\" at index "+strcnt+" in \""+s+"\" not valid with base "+fbase);
					found = true;
					val += chloc;
					break;
				}
			if (!found)
				throw new NumberFormatException( "Character \""+c+"\" at index "+strcnt+" in \""+s+"\" not valid with base "+fbase);
		}
		return val;
	}
	
	
	// En wrapper omkring parseString, der returnerer en int i stedet - se parseString
	public int parseInt( String s ) throws NumberFormatException
	{
		return (int) parseString( s );
	}
	
	// En wrapper omkring parseString - se parseString
	public long parseLong( String s ) throws NumberFormatException
	{
		return parseString( s );
	}
	
}
