////////////////////////////////////////
// 49104 v.12 opg. 1
//
// Et program til demonstration af et klasse-hieraki
//

import java.awt.*;
import java.applet.*;


//---------------------------------------------------------------------------------------
// Rektangel
class Rektangel
{
	protected int fX=0, fY=0, fH=0, fB=0;
	
	public Rektangel( int X, int Y, int B, int H )
	{
		fX = X;
		fY = Y;
		fH = H;
		fB = B;
	}
	
	public String ID()
	{
		return "Rektangel";
	}
	
	public void tegn( Graphics g )
	{
		g.setColor( new Color( 0, 0, 0 ) );
		g.drawRect( fX, fY, fB, fH );
		g.drawString( ID(), fX + fB + 10, fY +fH );
	}
		
		

}


//---------------------------------------------------------------------------------------
// Plade - Rektangel
class Plade extends Rektangel
{
	public final static int INTET  = 0;
	public final static int GULD   = 1;
	public final static int SOELV  = 2;
	public final static int STAAL  = 3;
	public final static int JERN   = 4;
	public final static int KOBBER = 5;
	
	protected final static String MatNavne[] = { "INTET", "GULD", "SOELV", "STAAL", "JERN", "KOBBER" };
	protected int fMat = 0;
	
	public Plade( int X, int Y, int B, int H, int Materiale )
	{
		super( X, Y, B, H );
		fMat = Materiale;
	}
	
	public String ID()
	{
		return "Plade";
	}
	
	public void tegn( Graphics g )
	{
		Color c;
		
		super.tegn( g );
		switch (fMat )
		{
			case INTET : c = Color.white; break;
			case SOELV : c = Color.lightGray; break;
			case GULD  : c = Color.magenta; break;
			case STAAL : c = Color.gray; break;
			case JERN  : c = Color.darkGray; break;
			case KOBBER: c = Color.red; break;
			
			default : c = Color.black; break;
		}
		
		g.setColor( c );			
		g.fillRect( fX+1, fY+1, fB-1, fH-1 );
	}
	
	
		
		
}



//---------------------------------------------------------------------------------------
// FarvetPlade - Plade - Rektangel
class FarvetPlade extends Plade
{
	protected Color fMaling;
	
	public FarvetPlade( int X, int Y, int B, int H, int Materiale, Color Maling )
	{
		super( X, Y, B, H, Materiale );
		fMaling = Maling;
	}
	
	public String ID()
	{
		return "Farvet Plade";
	}
	
	public void tegn( Graphics g )
	{
		super.tegn( g );
		g.setColor( fMaling );
		g.fillRect( fX+1, fY+1, fB-1, fH-1 );
	}
	
	
}


//---------------------------------------------------------------------------------------
// TekstetFarvetPlade - FarvetPlade - Plade - Rektangel
class TekstetFarvetPlade extends FarvetPlade
{
	protected String fTekst;
	protected Color fTekstFarve;
	
	public TekstetFarvetPlade( int X, int Y, int B, int H, int Materiale, Color Maling, String PladeTekst, Color TekstFarve )
	{
		super( X, Y, B, H, Materiale, Maling );
		fTekst = PladeTekst;
		fTekstFarve = TekstFarve;
	}
	
	public String ID()
	{
		return "Farvet Plade med tekst \""+fTekst+"\"";
	}
	
	public void tegn( Graphics g )
	{
		super.tegn( g );
		g.setColor( fTekstFarve );
		g.drawString( fTekst,
		              fX+ ((fB-g.getFontMetrics().stringWidth( fTekst ) )/2),
		              fY+fH - (( fH - (g.getFontMetrics().getMaxAscent()) )/2)-1 );
	}
	
}



//---------------------------------------------------------------------------------------
// Gadenavneskilt - TekstetFarvetPlade - FarvetPlade - Plade - Rektangel
class Gadenavneskilt extends TekstetFarvetPlade
{
	public Gadenavneskilt( int X, int Y, String Gadenavn )
	{
		super( X, Y, 0, 0, Plade.JERN, new Color( 0, 0, 128 ), Gadenavn, Color.white );
	}
	
	public String ID()
	{
		return "Gadenavneskilt i Null-erd";
	}
	
	public void tegn( Graphics g )
	{
		fH = g.getFontMetrics().getHeight() + 10;
		fB = g.getFontMetrics().stringWidth( fTekst ) + 10;
		super.tegn( g );
	}
	
		

}



//---------------------------------------------------------------------------------------
// Nummerplade - TekstetFarvetPlade - FarvetPlade - Plade - Rektangel
class Nummerplade extends TekstetFarvetPlade
{
	public static final int UREGISTRERET = 0;
	public static final int PRIVAT       = 1;
	public static final int FIRMA        = 2;
	public static final int DIPLOMAT     = 3;
	protected static final String Arter[] = {"Uregistreret","Privat", "Varevogn", "Diplomatsnylter" };
	
	protected int fArt;
	
	public Nummerplade( int X, int Y, int Type, String Nummer )
	{
		super( X, Y, 100, 20, Plade.JERN, Color.white, Nummer, Color.white );
		
		Color t, p;
				
		switch (Type)
		{
			case UREGISTRERET : p = Color.white; t = Color.white; break;
			case PRIVAT       : p = Color.white; t = Color.black; break;
			case FIRMA        : p = Color.yellow; t = Color.black; break;					
			case DIPLOMAT     : p = Color.blue; t = Color.white; break;
			default : p = Color.pink; t = Color.magenta; break;
		}
		
		fMaling = p;
		fTekstFarve = t;
		fArt = Type;
		
		
	}
	
	public String ID()
	{
		return Arter[fArt]+"-nummerplade i Timbuktistan";
	}
	
}


//---------------------------------------------------------------------------------------
// Hierakitest
public class Hieraki extends Applet
{
	public Rektangel figurer[] = new Rektangel[10];

	public void init()
	{
		int a = 40;
		
	
		resize( 500, 500 );
		figurer[ 0 ] = new Rektangel(10,a*1,100,20);
		figurer[ 1 ] = new Plade(10,a*2,100,20,Plade.STAAL );
		figurer[ 2 ] = new FarvetPlade(10,a*3,100,20,Plade.KOBBER, Color.pink);
  		figurer[ 3 ] = new TekstetFarvetPlade(10,a*4,100,20,Plade.GULD, Color.magenta, "Hejdu inc.", Color.black );
  		figurer[ 4 ] = new Gadenavneskilt(10,a*5, "ExceptionRoad");
  		figurer[ 5 ] = new Nummerplade(10,a*6, Nummerplade.PRIVAT, "AB 20 773");
  		figurer[ 6 ] = new Nummerplade(10,a*7, Nummerplade.FIRMA, "PQ 53 753");
  		figurer[ 7 ] = new Nummerplade(10,a*8, Nummerplade.DIPLOMAT, "JEG ER SNYLTER");
		figurer[ 8 ] = new Nummerplade(10,a*9, Nummerplade.UREGISTRERET, "JEG ER BLIND");
		figurer[ 9 ] = new TekstetFarvetPlade(10,a*10,200,60,Plade.SOELV, Color.orange, "Er jeg en fplade?", Color.black);
	}
	
	public void paint( Graphics g )
	{
		Font f = new Font("SansSerif", Font.PLAIN, 14 );
		g.setFont( f );
		for (int lp1=0; lp1< figurer.length; lp1++)
			figurer[ lp1 ].tegn( g );
	}
	
}

	
	
/*EOF*/